from github import Auth
from github import Github
import os
import sys
import random
import datetime
import time
import logging

logging.basicConfig(
    format="%(asctime)s.%(msecs)d %(levelname)-8s"
    + " [%(filename)s:%(lineno)d] %(message)s",
    datefmt="%Y-%m-%d:%H:%M:%S",
    level=logging.INFO,
)

# - return workflow status as exit code

TOKEN = os.environ.get("TOKEN")
SERVER = "https://github.gwd.broadcom.net"
REPO = "SED/ESG-ProdSec-Scanner"
BRANCH = "test-actions"
WORKFLOW_FILE = "scan.yml"
SCAN_IMAGE_LIST = os.environ.get("SCAN_IMAGE_LIST")

MAX_AGE_MINUTES = 10
RETRIES = 6
DELAY_SECONDS = 30

EXIT_NOT_TRIGGERED = 101
EXIT_RUN_ID_NOT_FOUND = 102
EXIT_RUN_FAIL = 103


def connect_to_github():
    auth = Auth.Token(TOKEN)
    g = Github(auth=auth, base_url=f"{SERVER}/api/v3")

    return g


def trigger_workflow_run(wf, SCAN_IMAGE_LIST, CUSTOM_ID):

    logging.info(f"Triggering workflow {wf} with CUSTOM_ID='{CUSTOM_ID}' for '{SCAN_IMAGE_LIST}'")

    return wf.create_dispatch(
        ref=BRANCH,
        inputs={
            "SCAN_IMAGE_LIST_INPUT": SCAN_IMAGE_LIST,
            "CUSTOM_ID": f"{CUSTOM_ID}"
        }
    )


def get_workflow_run(wf, CUSTOM_ID):

    logging.info(f"Searching for workflow run with CUSTOM_ID='{CUSTOM_ID}'")

    now = datetime.datetime.now()
    no_sooner_than = now - datetime.timedelta(minutes=MAX_AGE_MINUTES)
    runs = wf.get_runs(branch=BRANCH, created=f">{no_sooner_than.isoformat().split('.')[0]}")

    for run in runs:
        try:
            steps = run.jobs()[0].steps
            if steps[1].name == f"id={CUSTOM_ID}":
                logging.info(f"Found workflow with id: {run.id}")
                return run
        except Exception as e:
            pass

    logging.info(f"Workflow run not found (yet)")
    return None


if __name__ == "__main__":
    g = connect_to_github()

    r = g.get_repo(REPO)

    wf = r.get_workflow(WORKFLOW_FILE)

    CUSTOM_ID = random.randbytes(16).hex()

    # Trigger workflow run
    if trigger_workflow_run(wf, SCAN_IMAGE_LIST, CUSTOM_ID) is False:
        logging.error(f"Failed to trigger workflow {wf} with CUSTOM_ID='{CUSTOM_ID}' for '{SCAN_IMAGE_LIST}'")
        sys.exit(EXIT_NOT_TRIGGERED)

    for i in range(RETRIES):
        # Get workflow run
        run = get_workflow_run(wf, CUSTOM_ID)

        if run is not None:
            break

        logging.info(f"Waiting for {DELAY_SECONDS} seconds before trying again")
        time.sleep(DELAY_SECONDS)

    if run is None:
        logging.error("Workflow run not found!")
        sys.exit(EXIT_RUN_ID_NOT_FOUND)

    # Poll for status until completion
    logging.info(f"Polling for completion every {DELAY_SECONDS} seconds")
    while run.status != "completed":
        time.sleep(DELAY_SECONDS)
        run.update()

    print(f"{run.url} completed with status: {run.conclusion}")
    print(f"Workflow run URL: {SERVER}/{REPO}/actions/runs/{run.id}")

    if run.conclusion != "success":
        sys.exit(EXIT_RUN_FAIL)
